use atsamd_hal::common::gpio::{
    v2::{
        pin::{PA02, PA06, PA07, PA08, PA09},
        Floating, Input, PullUp,
    },
    Pin, Port,
};
use embedded_hal::digital::v2::InputPin;
use streamdeck_common::{ButtonPins, ButtonState};

pub(crate) struct Buttons {
    pub(crate) d0: Pin<PA08, Input<Floating>>,
    pub(crate) d1: Pin<PA02, Input<Floating>>,
    pub(crate) d2: Pin<PA09, Input<Floating>>,
    pub(crate) d3: Pin<PA07, Input<Floating>>,
    pub(crate) d4: Pin<PA06, Input<Floating>>,
}

pub(crate) struct Pins {
    d0: Pin<PA08, Input<PullUp>>,
    d1: Pin<PA02, Input<PullUp>>,
    d2: Pin<PA09, Input<PullUp>>,
    d3: Pin<PA07, Input<PullUp>>,
    d4: Pin<PA06, Input<PullUp>>,
}

impl<'a> ButtonPins<'a, 5> for Pins {
    fn to_array(&'a self) -> [&'a dyn InputPin<Error = ()>; 5] {
        [&self.d0, &self.d1, &self.d2, &self.d3, &self.d4]
    }
}

impl Buttons {
    pub(crate) fn init(self, port: &mut Port) -> ButtonState<Pins, 5> {
        ButtonState::from_pins(Pins {
            d0: self.d0.into_pull_up_input(port),
            d1: self.d1.into_pull_up_input(port),
            d2: self.d2.into_pull_up_input(port),
            d3: self.d3.into_pull_up_input(port),
            d4: self.d4.into_pull_up_input(port),
        })
    }
}
