use crate::device::Device;

static NAME: &'static [u8] = b"Streamdeck";
static AUTHOR: &'static [u8] = b"asonix";
static REPO: &'static [u8] = b"https://git.asonix.dog/asonix/trinket-streamdeck";

pub fn handle_input<D, const SN_LEN: usize>(
    device: &mut D,
    input: &[u8; 32],
    count: usize,
) -> Option<()>
where
    D: Device<SN_LEN>,
{
    if count == 5 && input.starts_with(b"ident") {
        let sn = device.serial_number();
        write_length_delim(device, &sn)?;
    } else if count == 4 && input.starts_with(b"name") {
        write_length_delim(device, NAME)?;
    } else if count == 4 && input.starts_with(b"repo") {
        write_length_delim(device, REPO)?;
    } else if count == 6 && input.starts_with(b"author") {
        write_length_delim(device, AUTHOR)?;
    } else if count == 5 && input.starts_with(b"reset") {
        device.reset();
    } else {
        write_length_delim(device, &input[..count])?;
    }

    Some(())
}

fn write_length_delim<D, const SN_LEN: usize>(device: &mut D, bytes: &[u8]) -> Option<()>
where
    D: Device<SN_LEN>,
{
    device.write(&[bytes.len() as u8])?;
    device.write(bytes)
}
